use async_trait::async_trait;
use std::path::PathBuf;
use thiserror::Error;

#[derive(Debug, Error)]
pub enum PackagerError {
  #[error("Internal packager error")]
  InternalBundlePackagerError {
    #[from]
    source: Box<dyn std::error::Error + Sync + Send>,
  },
  #[error(transparent)]
  IOError {
    #[from]
    source: std::io::Error,
  },
}

pub type Result<T, E = PackagerError> = std::result::Result<T, E>;

#[async_trait(?Send)]
pub trait BundlePackager {
  async fn generate(&self, path: PathBuf) -> Result<()>;
}
