use super::Bundle;
use async_trait::async_trait;
use std::path::PathBuf;
use thiserror::Error;

#[derive(Debug, Error)]
pub enum PollerError {
  #[error("Bundle ID is missing")]
  MissingBundleID,
  #[error("Internal poller error")]
  InternalPollerError {
    #[from]
    source: Box<dyn std::error::Error + Sync + Send>,
  },
  #[error(
    "Original Bundle ID {} does not match Current ID {}",
    original_id,
    current_id
  )]
  MismatchedBundleID {
    original_id: String,
    current_id: String,
  },
  #[error("Unable to unpack bundle")]
  UnpackError {
    #[source]
    source: std::io::Error,
  },
}

pub type Result<T, E = PollerError> = std::result::Result<T, E>;

pub enum PollResult {
  Skip,
  UpdateReady { etag: String },
  StaticUpdateReady { etag: Option<String>, path: PathBuf },
}

#[async_trait]
pub trait BundlePoller {
  async fn poll(&self, active_bundle: &Option<Bundle>) -> Result<PollResult>;

  async fn retrieve(&self, bundle_id: &str, path: PathBuf) -> Result<()>;
}
