use std::path::PathBuf;

use clap::{Parser, Subcommand};
use collective::{
  cli::{AppOpts, ConfigurableAppOpts},
  config::ConfigFileFormat,
};
use espresso::config::Config;

#[derive(Parser)]
#[clap(version, author = "Eduardo T. <ed@trujillo.io>")]
pub struct Opts {
  /// Sets a custom config file.
  #[clap(short, long)]
  config: Option<PathBuf>,
  /// A level of verbosity, and can be used multiple times
  #[clap(short, long, parse(from_occurrences))]
  verbose: i32,
  #[clap(subcommand)]
  pub subcmd: SubCommand,
}

#[derive(Subcommand)]
pub enum SubCommand {
  /// Start the server
  Serve,
  Bundle(BundleOpts),
}

/// Create a bundle using the configured bundler
#[derive(Parser)]
pub struct BundleOpts {
  /// The path to use a source for the bundle.
  pub source_path: PathBuf,
}

impl AppOpts for Opts {
  fn get_log_level_filter(&self) -> Option<log::LevelFilter> {
    match self.verbose {
      3 => Some(log::LevelFilter::Trace),
      2 => Some(log::LevelFilter::Debug),
      1 => Some(log::LevelFilter::Info),
      _ => None,
    }
  }
}

impl ConfigurableAppOpts<Config> for Opts {
  fn get_additional_config_paths(&self) -> Vec<(PathBuf, Option<ConfigFileFormat>)> {
    if let Some(config_path) = &self.config {
      vec![(config_path.clone(), None)]
    } else {
      vec![]
    }
  }
}
