use actix_files::NamedFile;
use actix_web::http::header::ContentDisposition;

use super::path_context::PathContext;

pub trait NamedFileExt {
  fn with_path_context(self, path_context: &PathContext) -> Self;
}

impl NamedFileExt for NamedFile {
  fn with_path_context(self, path_context: &PathContext) -> Self {
    let mut this = self;

    if let Some(mime_disposition) = path_context.get_mime_disposition() {
      if let Some(new_disposition_type) = mime_disposition.get(this.content_type()) {
        let original_parameters = this.content_disposition().parameters.clone();

        this = this.set_content_disposition(ContentDisposition {
          disposition: new_disposition_type.clone(),
          parameters: original_parameters,
        });
      }
    }

    this
      .use_etag(path_context.get_use_etag_or_default())
      .use_last_modified(path_context.get_use_last_modifiedor_default())
  }
}
